@@ -39,11 +39,18 @@ my @DOC_TAGS = qw(
   r3.1.3
   r3.1.4
   r3.2.0
+  r3.2.1
   r3.3.0
   r3.3.1
+  r3.3.2
   r3.4.0
   r3.4.1
+  r3.4.2
   r3.5.0
+  r3.5.1
+  r3.6.0-rc0
+  r3.6.0
+  r3.6.1
 );
 
 sub main {
