@@ -12,23 +12,26 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "catch.hpp"
-
 #include <cstring>
 
-#include <bson.h>
 #include <bsoncxx/builder/basic/array.hpp>
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/builder/core.hpp>
 #include <bsoncxx/builder/stream/array.hpp>
 #include <bsoncxx/builder/stream/document.hpp>
+#include <bsoncxx/exception/exception.hpp>
 #include <bsoncxx/json.hpp>
+#include <bsoncxx/private/libbson.hh>
+#include <bsoncxx/string/to_string.hpp>
+#include <bsoncxx/test_util/catch.hh>
 #include <bsoncxx/types.hpp>
-#include <bsoncxx/types/value.hpp>
+#include <bsoncxx/types/bson_value/view.hpp>
+
+namespace {
+
+using namespace bsoncxx;
 
 void bson_eq_stream(const bson_t* bson, const bsoncxx::builder::stream::document& builder) {
-    using namespace bsoncxx;
-
     bsoncxx::document::view expected(bson_get_data(bson), bson->len);
     bsoncxx::document::view test(builder.view());
 
@@ -40,8 +43,6 @@ void bson_eq_stream(const bson_t* bson, const bsoncxx::builder::stream::document
 
 template <typename T, typename U>
 void viewable_eq_viewable(const T& stream, const U& basic) {
-    using namespace bsoncxx;
-
     bsoncxx::document::view expected(stream.view());
     bsoncxx::document::view test(basic.view());
 
@@ -51,8 +52,6 @@ void viewable_eq_viewable(const T& stream, const U& basic) {
     REQUIRE(std::memcmp(expected.data(), test.data(), expected.length()) == 0);
 }
 
-using namespace bsoncxx;
-
 TEST_CASE("builder appends utf8", "[bsoncxx::builder::stream]") {
     bson_t expected;
     bson_init(&expected);
@@ -257,8 +256,12 @@ TEST_CASE("builder appends regex", "[bsoncxx::builder::stream]") {
     builder::stream::document b;
 
     bson_append_regex(&expected, "foo", -1, "^foo|bar$", "i");
+    bson_append_regex(&expected, "boo", -1, "^boo|far$", "");
+    bson_append_regex(&expected, "bar", -1, "^bar|foo$", "");
 
     b << "foo" << types::b_regex{"^foo|bar$", "i"};
+    b << "boo" << types::b_regex{"^boo|far$", ""};
+    b << "bar" << types::b_regex{"^bar|foo$"};
 
     bson_eq_stream(&expected, b);
 
@@ -337,13 +340,13 @@ TEST_CASE("builder appends int32", "[bsoncxx::builder::stream]") {
 }
 
 TEST_CASE("builder appends timestamp", "[bsoncxx::builder::stream]") {
+    builder::stream::document b;
+    types::b_timestamp foo{100, 1000};
+    b << "foo" << foo;
+
     bson_t expected;
     bson_init(&expected);
-    builder::stream::document b;
-
-    bson_append_timestamp(&expected, "foo", -1, 100, 1000);
-
-    b << "foo" << types::b_timestamp{100, 1000};
+    bson_append_timestamp(&expected, "foo", -1, foo.timestamp, foo.increment);
 
     bson_eq_stream(&expected, b);
 
@@ -394,9 +397,9 @@ TEST_CASE("builder appends decimal128", "[bsoncxx::builder::stream]") {
         bson_eq_stream(&expected, b);
     }
 
-    SECTION("bsoncxx::types::value with decimal128 works") {
+    SECTION("bsoncxx::types::bson_value::view with decimal128 works") {
         auto d = types::b_decimal128{"-1234E+999"};
-        auto v = types::value{d};
+        auto v = types::bson_value::view{d};
 
         REQUIRE(v.get_decimal128() == d);
 
@@ -628,8 +631,8 @@ TEST_CASE("builder appends lambdas", "[bsoncxx::builder::stream]") {
                   << "value1"
                   << "key2"
                   << "value2";
-            } << close_document << "c" << open_array << [](array_context<> a) { a << 1 << 2 << 3; }
-               << close_array;
+            } << close_document
+               << "c" << open_array << [](array_context<> a) { a << 1 << 2 << 3; } << close_array;
     }
 
     viewable_eq_viewable(expected, stream);
@@ -674,7 +677,7 @@ TEST_CASE("document core builder ownership", "[bsoncxx::builder::core]") {
 
     SECTION("when passing a stdx::string_view, ownership handled by caller") {
         std::string key{"sabich"};
-        stdx::string_view key_view{key};
+        stdx::string_view key_view(key);
         b.key_view(key_view);
         b.append(1);
     }
@@ -738,6 +741,85 @@ TEST_CASE("core builder open/close works", "[bsoncxx::builder::core]") {
     }
 }
 
+TEST_CASE("core view/extract methods throw when called with wrong top-level type",
+          "[bsoncxx::builder::core]") {
+    builder::core core_array(true);
+    builder::core core_document(false);
+
+    SECTION("view_array only throws when called on document") {
+        REQUIRE_NOTHROW(core_array.view_array());
+        REQUIRE_THROWS(core_document.view_array());
+    }
+
+    SECTION("extract_array only throws when called on document") {
+        REQUIRE_NOTHROW(core_array.extract_array());
+        REQUIRE_THROWS(core_document.extract_array());
+    }
+
+    SECTION("view_document only throws when called on array") {
+        REQUIRE_THROWS(core_array.view_document());
+        REQUIRE_NOTHROW(core_document.view_document());
+    }
+
+    SECTION("extract_document only throws when called on array") {
+        REQUIRE_THROWS(core_array.extract_document());
+        REQUIRE_NOTHROW(core_document.extract_document());
+    }
+}
+
+TEST_CASE("core builder throws on consecutive keys", "[bsoncxx::builder::core]") {
+    SECTION("appending key_view twice") {
+        builder::core builder{false};
+        REQUIRE_NOTHROW(builder.key_view("foo"));
+        REQUIRE_THROWS_AS(builder.key_view("bar"), bsoncxx::exception);
+    }
+
+    SECTION("appending key_view then key_owned") {
+        builder::core builder{false};
+        REQUIRE_NOTHROW(builder.key_view("foo"));
+        REQUIRE_THROWS_AS(builder.key_owned("bar"), bsoncxx::exception);
+    }
+
+    SECTION("appending key_owned then key_view") {
+        builder::core builder{false};
+        REQUIRE_NOTHROW(builder.key_owned("foo"));
+        REQUIRE_THROWS_AS(builder.key_view("bar"), bsoncxx::exception);
+    }
+
+    SECTION("appending key_owned twice") {
+        builder::core builder{false};
+        REQUIRE_NOTHROW(builder.key_owned("foo"));
+        REQUIRE_THROWS_AS(builder.key_owned("bar"), bsoncxx::exception);
+    }
+}
+
+TEST_CASE("core method chaining to build document works", "[bsoncxx::builder::core]") {
+    auto full_doc = builder::core{false}
+                        .key_owned("foo")
+                        .append(1)
+                        .key_owned("bar")
+                        .append(true)
+                        .extract_document();
+
+    REQUIRE(full_doc.view()["foo"].type() == types::b_int32::type_id);
+    REQUIRE(full_doc.view()["foo"].get_int32() == 1);
+    REQUIRE(full_doc.view()["bar"].type() == types::b_bool::type_id);
+    REQUIRE(full_doc.view()["bar"].get_bool() == true);
+}
+
+TEST_CASE("core method chaining to build array works", "[bsoncxx::builder::core]") {
+    auto array = builder::core{true}.append("foo").append(1).append(true).extract_array();
+    auto array_view = array.view();
+
+    REQUIRE(std::distance(array_view.begin(), array_view.end()) == 3);
+    REQUIRE(array_view[0].type() == type::k_utf8);
+    REQUIRE(string::to_string(array_view[0].get_string().value) == "foo");
+    REQUIRE(array_view[1].type() == type::k_int32);
+    REQUIRE(array_view[1].get_int32().value == 1);
+    REQUIRE(array_view[2].type() == type::k_bool);
+    REQUIRE(array_view[2].get_bool().value == true);
+}
+
 TEST_CASE("basic document builder works", "[bsoncxx::builder::basic]") {
     builder::stream::document stream;
     builder::basic::document basic;
@@ -787,15 +869,207 @@ TEST_CASE("basic document builder works", "[bsoncxx::builder::basic]") {
         {
             using namespace builder::basic;
 
-            basic.append(kvp("hello", "world"), kvp("foo", 35), kvp("bar", [](sub_document sd) {
-                sd.append(kvp("que", "qux"));
-            }), kvp("baz", [](sub_array sa) { sa.append(1, 2, 3); }));
+            basic.append(kvp("hello", "world"),
+                         kvp("foo", 35),
+                         kvp("bar", [](sub_document sd) { sd.append(kvp("que", "qux")); }),
+                         kvp("baz", [](sub_array sa) { sa.append(1, 2, 3); }));
         }
 
         viewable_eq_viewable(stream, basic);
     }
 }
 
+TEST_CASE("basic document builder move semantics work", "[bsoncxx::builder::basic::document]") {
+    using builder::basic::kvp;
+
+    builder::basic::document doc_base;
+    doc_base.append(kvp("a", "A"));
+    doc_base.append(kvp("b", "B"));
+    doc_base.append(kvp("bool", true));
+
+    SECTION("move constructor from the same scope") {
+        builder::basic::document doc;
+        doc.append(kvp("a", "A"));
+        doc.append(kvp("b", "B"));
+        doc.append(kvp("bool", true));
+
+        builder::basic::document doc_same_scope(std::move(doc));
+        viewable_eq_viewable(doc_same_scope, doc_base);
+
+        doc_same_scope.append(kvp("after", true));
+        doc_base.append(kvp("after", true));
+        viewable_eq_viewable(doc_same_scope, doc_base);
+    }
+
+    SECTION("move constructor from outer scope") {
+        builder::basic::document doc;
+        doc.append(kvp("a", "A"));
+        doc.append(kvp("b", "B"));
+        doc.append(kvp("bool", true));
+
+        {
+            builder::basic::document doc_different_scope(std::move(doc));
+            viewable_eq_viewable(doc_different_scope, doc_base);
+
+            doc_different_scope.append(kvp("ds", 12));
+            doc_base.append(kvp("ds", 12));
+            viewable_eq_viewable(doc_different_scope, doc_base);
+        }
+    }
+
+    SECTION("move assignment operator from the same scope") {
+        builder::basic::document doc;
+        doc.append(kvp("a", "A"));
+        doc.append(kvp("b", "B"));
+        doc.append(kvp("bool", true));
+        builder::basic::document doc_from_same_scope = std::move(doc);
+
+        viewable_eq_viewable(doc_from_same_scope, doc_base);
+
+        doc_from_same_scope.append(kvp("next", 12));
+        doc_base.append(kvp("next", 12));
+        viewable_eq_viewable(doc_from_same_scope, doc_base);
+    }
+
+    SECTION("move assignment operator from inner scope") {
+        builder::basic::document doc_from_inner_scope;
+
+        {
+            builder::basic::document doc;
+            doc.append(kvp("a", "A"));
+            doc.append(kvp("b", "B"));
+            doc.append(kvp("bool", true));
+            doc_from_inner_scope = std::move(doc);
+        }
+
+        viewable_eq_viewable(doc_from_inner_scope, doc_base);
+
+        doc_from_inner_scope.append(kvp("next", 12));
+        doc_base.append(kvp("next", 12));
+        viewable_eq_viewable(doc_from_inner_scope, doc_base);
+    }
+
+    SECTION("move assignment operator from outer scope") {
+        builder::basic::document doc;
+        doc.append(kvp("a", "A"));
+        doc.append(kvp("b", "B"));
+        doc.append(kvp("bool", true));
+
+        {
+            builder::basic::document doc_from_outer_scope = std::move(doc);
+            viewable_eq_viewable(doc_from_outer_scope, doc_base);
+
+            doc_from_outer_scope.append(kvp("after", true));
+            doc_base.append(kvp("after", true));
+            viewable_eq_viewable(doc_from_outer_scope, doc_base);
+        }
+    }
+
+    SECTION("move assignment operator repeated calls") {
+        builder::basic::document doc_chain1;
+        doc_chain1.append(kvp("a", "A"));
+        doc_chain1.append(kvp("b", "B"));
+        doc_chain1.append(kvp("bool", true));
+
+        builder::basic::document doc_chain2 = std::move(doc_chain1);
+        builder::basic::document doc_chain3 = std::move(doc_chain2);
+        viewable_eq_viewable(doc_chain3, doc_base);
+    }
+}
+
+TEST_CASE("basic array builder move semantics work", "[bsoncxx::builder::basic::array]") {
+    builder::basic::array arr_base;
+    arr_base.append("a");
+    arr_base.append(false);
+
+    SECTION("move constructor from same scope") {
+        builder::basic::array arr;
+        arr.append("a");
+        arr.append(false);
+
+        builder::basic::array arr_from_same_scope(std::move(arr));
+        viewable_eq_viewable(arr_from_same_scope, arr_base);
+
+        arr_base.append(12);
+        arr_from_same_scope.append(12);
+        viewable_eq_viewable(arr_from_same_scope, arr_base);
+    }
+
+    SECTION("move constructor from outer scope") {
+        builder::basic::array arr;
+        arr.append("a");
+        arr.append(false);
+
+        {
+            builder::basic::array arr_from_outer_scope(std::move(arr));
+            viewable_eq_viewable(arr_from_outer_scope, arr_base);
+
+            arr_from_outer_scope.append("after");
+            arr_base.append("after");
+            viewable_eq_viewable(arr_from_outer_scope, arr_base);
+        }
+    }
+
+    SECTION("move assignment operator from same scope") {
+        builder::basic::array arr;
+        arr.append("a");
+        arr.append(false);
+
+        builder::basic::array arr_from_same_scope = std::move(arr);
+        viewable_eq_viewable(arr_from_same_scope, arr_base);
+
+        arr_base.append(12);
+        arr_from_same_scope.append(12);
+        viewable_eq_viewable(arr_from_same_scope, arr_base);
+    }
+
+    SECTION("move assignment operator from inner scope") {
+        builder::basic::array arr_from_inner_scope;
+
+        {
+            builder::basic::array arr;
+            arr.append("a");
+            arr.append(false);
+            arr_from_inner_scope = std::move(arr);
+        }
+
+        viewable_eq_viewable(arr_from_inner_scope, arr_base);
+
+        arr_from_inner_scope.append(12);
+        arr_base.append(12);
+        viewable_eq_viewable(arr_from_inner_scope, arr_base);
+    }
+
+    SECTION("move assignment operator from outer scope") {
+        builder::basic::array arr;
+        arr.append("a");
+        arr.append(false);
+
+        {
+            builder::basic::array arr_from_outer_scope = std::move(arr);
+            viewable_eq_viewable(arr_from_outer_scope, arr_base);
+
+            arr_from_outer_scope.append("after");
+            arr_base.append("after");
+            viewable_eq_viewable(arr_from_outer_scope, arr_base);
+        }
+    }
+
+    SECTION("move assignment operator repeated calls") {
+        builder::basic::array arr;
+        arr.append("a");
+        arr.append(false);
+
+        builder::basic::array arr_chain_1 = std::move(arr);
+        builder::basic::array arr_chain_2 = std::move(arr_chain_1);
+        viewable_eq_viewable(arr_chain_2, arr_base);
+
+        arr_chain_2.append(12);
+        arr_base.append(12);
+        viewable_eq_viewable(arr_chain_2, arr_base);
+    }
+}
+
 TEST_CASE("basic array builder works", "[bsoncxx::builder::basic]") {
     using namespace builder::basic;
 
@@ -908,7 +1182,30 @@ TEST_CASE("array::view works", "[bsoncxx::builder::array]") {
     REQUIRE(stream.view()[2].get_int32() == 98);
 }
 
-TEST_CASE("stream in a document::view works", "[bsoncxx::bulder::stream]") {
+TEST_CASE("builder::basic::make_document works", "[bsoncxx::builder::basic::make_document]") {
+    auto full_doc = builder::basic::make_document(builder::basic::kvp("foo", 1),
+                                                  builder::basic::kvp("bar", true));
+
+    REQUIRE(full_doc.view()["foo"].type() == types::b_int32::type_id);
+    REQUIRE(full_doc.view()["foo"].get_int32() == 1);
+    REQUIRE(full_doc.view()["bar"].type() == types::b_bool::type_id);
+    REQUIRE(full_doc.view()["bar"].get_bool() == true);
+}
+
+TEST_CASE("builder::basic::make_array works", "[bsoncxx::builder::basic::make_array]") {
+    auto array = builder::basic::make_array("foo", 1, true);
+    auto array_view = array.view();
+
+    REQUIRE(std::distance(array_view.begin(), array_view.end()) == 3);
+    REQUIRE(array_view[0].type() == type::k_utf8);
+    REQUIRE(string::to_string(array_view[0].get_string().value) == "foo");
+    REQUIRE(array_view[1].type() == type::k_int32);
+    REQUIRE(array_view[1].get_int32().value == 1);
+    REQUIRE(array_view[2].type() == type::k_bool);
+    REQUIRE(array_view[2].get_bool().value == true);
+}
+
+TEST_CASE("stream in a document::view works", "[bsoncxx::builder::stream]") {
     using namespace builder::stream;
 
     auto sub_doc = builder::stream::document{} << "b" << 1 << finalize;
@@ -919,7 +1216,7 @@ TEST_CASE("stream in a document::view works", "[bsoncxx::bulder::stream]") {
     REQUIRE(full_doc.view()["a"]["b"].get_int32().value == 1);
 }
 
-TEST_CASE("stream in an array::view works", "[bsoncxx::bulder::stream]") {
+TEST_CASE("stream in an array::view works", "[bsoncxx::builder::stream]") {
     using namespace builder::stream;
 
     auto sub_array = builder::stream::array{} << 1 << 2 << 3 << finalize;
@@ -929,3 +1226,12 @@ TEST_CASE("stream in an array::view works", "[bsoncxx::bulder::stream]") {
     REQUIRE(full_doc.view()["a"][1].type() == bsoncxx::types::b_int32::type_id);
     REQUIRE(full_doc.view()["a"][1].get_int32().value == 2);
 }
+
+TEST_CASE("builder::stream::document throws on consecutive keys", "[bsoncxx::builder::core]") {
+    builder::stream::document doc;
+    REQUIRE_NOTHROW(doc << "foo"
+                        << "bar");
+    REQUIRE_NOTHROW(doc << "far");
+    REQUIRE_THROWS_AS(doc << "boo", bsoncxx::exception);
+}
+}  // namespace
