@@ -22,7 +22,6 @@
 #include <bsoncxx/json.hpp>
 #include <bsoncxx/private/helpers.hh>
 #include <bsoncxx/private/libbson.hh>
-#include <bsoncxx/stdx/make_unique.hpp>
 #include <bsoncxx/stdx/optional.hpp>
 #include <bsoncxx/test_util/catch.hh>
 #include <mongocxx/client.hpp>
@@ -31,7 +30,6 @@
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/exception/operation_exception.hpp>
 #include <mongocxx/instance.hpp>
-#include <mongocxx/options/index.hpp>
 #include <mongocxx/options/update.hpp>
 #include <mongocxx/pipeline.hpp>
 #include <mongocxx/private/conversions.hh>
@@ -118,7 +116,7 @@ TEST_CASE("Collection", "[collection]") {
                 bsoncxx::document::view o(bson_get_data(options), options->len);
 
                 bsoncxx::stdx::string_view bar(
-                    p[0].get_document().value["$match"].get_document().value["foo"].get_utf8());
+                    p[0].get_document().value["$match"].get_document().value["foo"].get_string());
                 std::int32_t one(
                     p[1].get_document().value["$sort"].get_document().value["foo"].get_int32());
 
@@ -329,7 +327,7 @@ TEST_CASE("Collection", "[collection]") {
         mongocxx::stdx::optional<bool> expected_allow_partial_results;
         mongocxx::stdx::optional<bsoncxx::stdx::string_view> expected_comment{};
         mongocxx::stdx::optional<mongocxx::cursor::type> expected_cursor_type{};
-        mongocxx::stdx::optional<bsoncxx::types::value> expected_hint{};
+        mongocxx::stdx::optional<bsoncxx::types::bson_value::view> expected_hint{};
         mongocxx::stdx::optional<bool> expected_no_cursor_timeout;
         mongocxx::stdx::optional<bsoncxx::document::view> expected_sort{};
         mongocxx::stdx::optional<read_preference> expected_read_preference{};
@@ -350,7 +348,7 @@ TEST_CASE("Collection", "[collection]") {
                         *expected_allow_partial_results);
             }
             if (expected_comment) {
-                REQUIRE(opts_view["comment"].get_utf8().value == *expected_comment);
+                REQUIRE(opts_view["comment"].get_string().value == *expected_comment);
             }
             if (expected_cursor_type) {
                 bsoncxx::document::element tailable = opts_view["tailable"];
@@ -371,7 +369,7 @@ TEST_CASE("Collection", "[collection]") {
                 }
             }
             if (expected_hint) {
-                REQUIRE(opts_view["hint"].get_utf8() == expected_hint->get_utf8());
+                REQUIRE(opts_view["hint"].get_string() == expected_hint->get_string());
             }
             if (expected_no_cursor_timeout) {
                 REQUIRE(opts_view["noCursorTimeout"].get_bool().value ==
@@ -468,8 +466,7 @@ TEST_CASE("Collection", "[collection]") {
         auto expected_order_setting = false;
         auto expect_set_bypass_document_validation_called = false;
         auto expected_bypass_document_validation = false;
-        bool bulk_operation_op_called = false;
-        bool bulk_operation_destroy_called = false;
+        mongocxx::stdx::optional<bsoncxx::types::bson_value::view> expected_hint{};
 
         auto modification_doc = make_document(kvp("cool", "wow"), kvp("foo", "bar"));
 
@@ -600,6 +597,12 @@ TEST_CASE("Collection", "[collection]") {
                     }
                 }
 
+                if (expected_hint) {
+                    REQUIRE(options_view["hint"].get_string() == expected_hint->get_string());
+                } else {
+                    REQUIRE(!options_view["hint"]);
+                }
+
                 return true;
             });
 
@@ -619,6 +622,12 @@ TEST_CASE("Collection", "[collection]") {
                 options.upsert(upsert_option);
             }
 
+            SECTION("With hint") {
+                hint index_hint("a_1");
+                expected_hint = index_hint.to_value();
+                options.hint(index_hint);
+            }
+
             SECTION("With bypass_document_validation") {
                 expect_set_bypass_document_validation_called = true;
                 expected_bypass_document_validation = true;
@@ -713,6 +722,12 @@ TEST_CASE("Collection", "[collection]") {
                     }
                 }
 
+                if (expected_hint) {
+                    REQUIRE(options_view["hint"].get_string() == expected_hint->get_string());
+                } else {
+                    REQUIRE(!options_view["hint"]);
+                }
+
                 return true;
             });
 
@@ -732,6 +747,12 @@ TEST_CASE("Collection", "[collection]") {
                 options.upsert(upsert_option);
             }
 
+            SECTION("With hint") {
+                hint index_hint("a_1");
+                expected_hint = index_hint.to_value();
+                options.hint(index_hint);
+            }
+
             mongo_coll.update_many(filter_doc.view(), modification_doc.view(), options);
             REQUIRE(bulk_operation_execute_called);
             perform_checks();
@@ -784,6 +805,12 @@ TEST_CASE("Collection", "[collection]") {
                     }
                 }
 
+                if (expected_hint) {
+                    REQUIRE(options_view["hint"].get_string() == expected_hint->get_string());
+                } else {
+                    REQUIRE(!options_view["hint"]);
+                }
+
                 return true;
             });
 
@@ -803,6 +830,12 @@ TEST_CASE("Collection", "[collection]") {
                 options.upsert(upsert_option);
             }
 
+            SECTION("With hint") {
+                hint index_hint("a_1");
+                expected_hint = index_hint.to_value();
+                options.hint(index_hint);
+            }
+
             mongo_coll.replace_one(filter_doc.view(), modification_doc.view(), options);
             REQUIRE(bulk_operation_execute_called);
             perform_checks();
@@ -828,14 +861,29 @@ TEST_CASE("Collection", "[collection]") {
 
         SECTION("Delete One", "[collection::delete_one]") {
             expected_order_setting = true;
-            bulk_operation_remove_one_with_opts->interpose(
-                [&](mongoc_bulk_operation_t*, const bson_t* doc, const bson_t*, bson_error_t*) {
-                    bulk_operation_op_called = true;
-                    REQUIRE(bson_get_data(doc) == filter_doc.view().data());
-                    return true;
-                });
+            bulk_operation_remove_one_with_opts->interpose([&](
+                mongoc_bulk_operation_t*, const bson_t* doc, const bson_t* options, bson_error_t*) {
+                bulk_operation_op_called = true;
+                REQUIRE(bson_get_data(doc) == filter_doc.view().data());
 
-            mongo_coll.delete_one(filter_doc.view());
+                bsoncxx::document::view options_view{bson_get_data(options), options->len};
+                if (expected_hint) {
+                    CAPTURE(to_json(options_view));
+                    REQUIRE(options_view["hint"].get_string() == expected_hint->get_string());
+                } else {
+                    REQUIRE(!options_view["hint"]);
+                }
+                return true;
+            });
+
+            options::delete_options options;
+            SECTION("With hint") {
+                hint index_hint("a_1");
+                expected_hint = index_hint.to_value();
+                options.hint(index_hint);
+            }
+
+            mongo_coll.delete_one(filter_doc.view(), options);
             REQUIRE(bulk_operation_execute_called);
             perform_checks();
         }
@@ -856,14 +904,29 @@ TEST_CASE("Collection", "[collection]") {
 
         SECTION("Delete Many", "[collection::delete_many]") {
             expected_order_setting = true;
-            bulk_operation_remove_many_with_opts->interpose(
-                [&](mongoc_bulk_operation_t*, const bson_t* doc, const bson_t*, bson_error_t*) {
-                    bulk_operation_op_called = true;
-                    REQUIRE(bson_get_data(doc) == filter_doc.view().data());
-                    return true;
-                });
+            bulk_operation_remove_many_with_opts->interpose([&](
+                mongoc_bulk_operation_t*, const bson_t* doc, const bson_t* options, bson_error_t*) {
+                bulk_operation_op_called = true;
+                REQUIRE(bson_get_data(doc) == filter_doc.view().data());
 
-            mongo_coll.delete_many(filter_doc.view());
+                bsoncxx::document::view options_view{bson_get_data(options), options->len};
+                if (expected_hint) {
+                    CAPTURE(to_json(options_view));
+                    REQUIRE(options_view["hint"].get_string() == expected_hint->get_string());
+                } else {
+                    REQUIRE(!options_view["hint"]);
+                }
+                return true;
+            });
+
+            options::delete_options options;
+            SECTION("With hint") {
+                hint index_hint("a_1");
+                expected_hint = index_hint.to_value();
+                options.hint(index_hint);
+            }
+
+            mongo_coll.delete_many(filter_doc.view(), options);
             REQUIRE(bulk_operation_execute_called);
             perform_checks();
         }
