@@ -14,7 +14,6 @@
 
 #include <cmath>
 #include <exception>
-#include <fstream>
 #include <functional>
 #include <memory>
 #include <string>
@@ -31,7 +30,7 @@
 #include <bsoncxx/stdx/string_view.hpp>
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/test_util/catch.hh>
-#include <bsoncxx/types/value.hpp>
+#include <bsoncxx/types/bson_value/view.hpp>
 #include <mongocxx/client.hpp>
 #include <mongocxx/collection.hpp>
 #include <mongocxx/cursor.hpp>
@@ -60,7 +59,7 @@ bsoncxx::stdx::optional<test_util::item_t> make_optional(test_util::item_t item)
 // Query the GridFS files collection and fetch the length of the file.
 //
 // Returns -1 if the file is not found.
-std::int64_t get_length_of_gridfs_file(gridfs::bucket bucket, types::value id) {
+std::int64_t get_length_of_gridfs_file(gridfs::bucket bucket, types::bson_value::view id) {
     builder::basic::document filter;
     filter.append(builder::basic::kvp("_id", id));
     cursor cursor = bucket.find(filter.extract());
@@ -95,7 +94,7 @@ bsoncxx::stdx::optional<test_util::item_t> transform_hex(test_util::item_t pair,
     }
 
     std::basic_string<std::uint8_t> bytes =
-        test_util::convert_hex_string_to_bytes(data["$hex"].get_utf8().value);
+        test_util::convert_hex_string_to_bytes(data["$hex"].get_string().value);
     types::b_binary binary_data = {
         binary_sub_type::k_binary, static_cast<std::uint32_t>(bytes.size()), bytes.data()};
 
@@ -130,8 +129,9 @@ bsoncxx::stdx::optional<test_util::item_t> convert_length_to_int64(test_util::it
 
     types::b_int64 length = {value.get_int32()};
 
-    return make_optional(std::make_pair(
-        bsoncxx::stdx::optional<bsoncxx::stdx::string_view>("length"), types::value{length}));
+    return make_optional(
+        std::make_pair(bsoncxx::stdx::optional<bsoncxx::stdx::string_view>("length"),
+                       types::bson_value::view{length}));
 }
 
 void compare_collections(database db) {
@@ -163,7 +163,7 @@ void compare_collections(database db) {
 
         REQUIRE(expected["filename"]);
         REQUIRE(actual["filename"]);
-        REQUIRE(expected["filename"].get_utf8().value == actual["filename"].get_utf8().value);
+        REQUIRE(expected["filename"].get_string().value == actual["filename"].get_string().value);
     }
 
     cursor expected_chunks_cursor = db["expected.chunks"].find({});
@@ -205,7 +205,7 @@ void test_download(database db,
     document::view arguments = operation["arguments"].get_document().value;
 
     REQUIRE(arguments["id"]);
-    types::value id = arguments["id"].get_value();
+    types::bson_value::view id = arguments["id"].get_value();
 
     // Allocate the space needed to store the result.
     std::int64_t length = get_length_of_gridfs_file(bucket, id);
@@ -216,7 +216,7 @@ void test_download(database db,
     }
 
     if (assert_doc["error"]) {
-        bsoncxx::stdx::string_view error = assert_doc["error"].get_utf8().value;
+        bsoncxx::stdx::string_view error = assert_doc["error"].get_string().value;
 
         // If the GridFS file is not found, an error should be thrown when the download stream is
         // opened.
@@ -244,7 +244,7 @@ void test_download(database db,
     // The GridFS spec specifies the expected binary data in the form of { $hex: "<hexadecimal
     // string>" }, which needs to be converted to an array of bytes.
     REQUIRE(result["$hex"]);
-    std::string hex = bsoncxx::string::to_string(result["$hex"].get_utf8().value);
+    std::string hex = bsoncxx::string::to_string(result["$hex"].get_string().value);
     std::basic_string<std::uint8_t> expected = test_util::convert_hex_string_to_bytes(hex);
 
     REQUIRE(actual_size == expected.size());
@@ -276,7 +276,7 @@ void test_upload(database db,
     }
 
     REQUIRE(arguments["filename"]);
-    bsoncxx::stdx::string_view filename = arguments["filename"].get_utf8().value;
+    bsoncxx::stdx::string_view filename = arguments["filename"].get_string().value;
 
     gridfs::uploader uploader = bucket.open_upload_stream(filename, upload_options);
 
@@ -284,7 +284,7 @@ void test_upload(database db,
     document::view source = arguments["source"].get_document().value;
 
     REQUIRE(source["$hex"]);
-    std::string hex = bsoncxx::string::to_string(source["$hex"].get_utf8().value);
+    std::string hex = bsoncxx::string::to_string(source["$hex"].get_string().value);
     std::basic_string<std::uint8_t> source_bytes = test_util::convert_hex_string_to_bytes(hex);
 
     uploader.write(source_bytes.data(), source_bytes.size());
@@ -326,7 +326,7 @@ void test_upload(database db,
                     return convert_length_to_int64(pair, context);
                 }
 
-                std::string id_str = bsoncxx::string::to_string(value.get_utf8().value);
+                std::string id_str = bsoncxx::string::to_string(value.get_string().value);
 
                 if (id_str == "*actual") {
                     return bsoncxx::stdx::optional<test_util::item_t>{};
@@ -336,7 +336,7 @@ void test_upload(database db,
                     return make_optional(pair);
                 }
 
-                return make_optional(std::make_pair(pair.first, types::value{id}));
+                return make_optional(std::make_pair(pair.first, types::bson_value::view{id}));
             });
 
         db.run_command(transformed_data.view());
@@ -353,7 +353,7 @@ void test_delete(database db,
     document::view arguments = operation["arguments"].get_document().value;
 
     REQUIRE(arguments["id"]);
-    types::value id = arguments["id"].get_value();
+    types::bson_value::view id = arguments["id"].get_value();
 
     if (assert_doc["error"]) {
         REQUIRE_THROWS_AS(bucket.delete_file(id), std::exception);
@@ -473,7 +473,8 @@ void run_gridfs_tests_in_file(std::string test_path, client* client) {
     gridfs::bucket bucket = db.gridfs_bucket();
 
     for (auto&& test : tests) {
-        std::string description = bsoncxx::string::to_string(test["description"].get_utf8().value);
+        std::string description =
+            bsoncxx::string::to_string(test["description"].get_string().value);
         INFO("Test description: " << description);
         initialize_collections(db, test_spec_view["data"].get_document().value);
 
@@ -490,7 +491,7 @@ void run_gridfs_tests_in_file(std::string test_path, client* client) {
 
         REQUIRE(act["operation"]);
         auto test_runner =
-            gridfs_test_runners[bsoncxx::string::to_string(act["operation"].get_utf8().value)];
+            gridfs_test_runners[bsoncxx::string::to_string(act["operation"].get_string().value)];
         test_runner(db, bucket, act, assert_doc);
     }
 }
