@@ -27,30 +27,48 @@ MONGOCXX_INLINE_NAMESPACE_BEGIN
 namespace spec {
 
 using namespace mongocxx;
+using bsoncxx::to_json;
 
 void apm_checker::compare(bsoncxx::array::view expectations,
                           bool allow_extra,
                           const test_util::match_visitor& match_visitor) {
-    using bsoncxx::to_json;
+    auto is_kill_cursor = [](bsoncxx::document::value v) {
+        return v.view()["command_started_event"]["command"]["killCursors"];
+    };
 
     auto events_iter = _events.begin();
+    if (_skip_kill_cursors)
+        _events.erase(std::remove_if(_events.begin(), _events.end(), is_kill_cursor));
     for (auto expectation : expectations) {
         REQUIRE(events_iter != _events.end());
+
         auto expected = expectation.get_document().view();
+        CAPTURE(to_json(*events_iter), expectation);
         REQUIRE_BSON_MATCHES_V(*events_iter, expected, match_visitor);
         events_iter++;
     }
-    if (!allow_extra) {
-        if (_skip_kill_cursors && events_iter != _events.end()) {
-            if (events_iter->view()["command_started_event"]["command"]["killCursors"]) {
-                ++events_iter;
-            }
-        }
-        // Then there must be no extra events after the expected ones.
-        REQUIRE(events_iter == _events.end());
+
+    REQUIRE((allow_extra || events_iter == _events.end()));
+}
+
+void apm_checker::has(bsoncxx::array::view expectations) {
+    for (auto expectation : expectations) {
+        auto expected = expectation.get_document().view();
+        CAPTURE(to_json(expected).c_str());
+        REQUIRE(std::find_if(_events.begin(), _events.end(), [&](bsoncxx::document::view doc) {
+                    return test_util::matches(doc, expected);
+                }) != _events.end());
     }
 }
 
+void apm_checker::print_all() {
+    printf("\n\n");
+    for (auto&& event : _events) {
+        printf("APM event: %s\n", bsoncxx::to_json(event).c_str());
+    }
+    printf("\n\n");
+}
+
 options::apm apm_checker::get_apm_opts(bool command_started_events_only) {
     options::apm opts;
     using namespace bsoncxx::builder::basic;
