@@ -14,16 +14,10 @@
 
 #include <mongocxx/test/spec/operation.hh>
 
-#include <fstream>
 #include <functional>
-#include <memory>
-#include <set>
-#include <sstream>
 #include <vector>
 
-#include <bsoncxx/builder/basic/array.hpp>
 #include <bsoncxx/builder/basic/document.hpp>
-#include <bsoncxx/exception/exception.hpp>
 #include <bsoncxx/json.hpp>
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/test_util/catch.hh>
@@ -32,10 +26,6 @@
 #include <mongocxx/collection.hpp>
 #include <mongocxx/cursor.hpp>
 #include <mongocxx/database.hpp>
-#include <mongocxx/exception/bulk_write_exception.hpp>
-#include <mongocxx/exception/error_code.hpp>
-#include <mongocxx/exception/logic_error.hpp>
-#include <mongocxx/exception/private/mongoc_error.hh>
 #include <mongocxx/options/aggregate.hpp>
 #include <mongocxx/options/count.hpp>
 #include <mongocxx/options/delete.hpp>
@@ -47,16 +37,13 @@
 #include <mongocxx/options/find_one_common_options.hpp>
 #include <mongocxx/options/replace.hpp>
 #include <mongocxx/options/update.hpp>
-#include <mongocxx/private/libbson.hh>
 #include <mongocxx/result/delete.hpp>
 #include <mongocxx/result/insert_many.hpp>
 #include <mongocxx/result/insert_one.hpp>
 #include <mongocxx/result/replace_one.hpp>
 #include <mongocxx/result/update.hpp>
-#include <mongocxx/test_util/client_helpers.hh>
 
 #include <mongocxx/config/private/prelude.hh>
-#include <third_party/catch/include/helpers.hpp>
 
 namespace mongocxx {
 MONGOCXX_INLINE_NAMESPACE_BEGIN
@@ -86,7 +73,7 @@ pipeline build_pipeline(array::view pipeline_docs) {
 bsoncxx::stdx::optional<read_concern> lookup_read_concern(document::view doc) {
     if (doc["readConcern"] && doc["readConcern"]["level"]) {
         read_concern rc;
-        rc.acknowledge_string(string::to_string(doc["readConcern"]["level"].get_utf8().value));
+        rc.acknowledge_string(string::to_string(doc["readConcern"]["level"].get_string().value));
         return rc;
     }
 
@@ -98,7 +85,7 @@ bsoncxx::stdx::optional<write_concern> lookup_write_concern(document::view doc)
         write_concern wc;
         document::element w = doc["writeConcern"]["w"];
         if (w.type() == bsoncxx::type::k_utf8) {
-            std::string level = string::to_string(w.get_utf8().value);
+            std::string level = string::to_string(w.get_string().value);
             if (level.compare("majority") == 0) {
                 wc.acknowledge_level(write_concern::level::k_majority);
             } else if (level.compare("acknowledged") == 0) {
@@ -118,7 +105,7 @@ bsoncxx::stdx::optional<write_concern> lookup_write_concern(document::view doc)
 bsoncxx::stdx::optional<read_preference> lookup_read_preference(document::view doc) {
     if (doc["readPreference"] && doc["readPreference"]["mode"]) {
         read_preference rp;
-        std::string mode = string::to_string(doc["readPreference"]["mode"].get_utf8().value);
+        std::string mode = string::to_string(doc["readPreference"]["mode"].get_string().value);
         if (mode.compare("Primary") == 0) {
             rp.mode(read_preference::read_mode::k_primary);
         } else if (mode.compare("PrimaryPreferred") == 0) {
@@ -148,7 +135,7 @@ client_session* operation_runner::_lookup_session(stdx::string_view key) {
 
 client_session* operation_runner::_lookup_session(document::view doc) {
     if (doc["session"]) {
-        stdx::string_view session_name = doc["session"].get_utf8().value;
+        stdx::string_view session_name = doc["session"].get_string().value;
         return _lookup_session(session_name);
     }
     return nullptr;
@@ -173,7 +160,7 @@ document::value operation_runner::_run_aggregate(document::view operation) {
     client_session* session = _lookup_session(operation["arguments"].get_document().value);
 
     if (operation["object"] &&
-        operation["object"].get_utf8().value == stdx::string_view{"database"}) {
+        operation["object"].get_string().value == stdx::string_view{"database"}) {
         REQUIRE(_db);
 
         // Run on the database
@@ -201,40 +188,6 @@ document::value operation_runner::_run_aggregate(document::view operation) {
     return result.extract();
 }
 
-document::value operation_runner::_run_count(document::view operation) {
-    document::view arguments = operation["arguments"].get_document().value;
-    document::view filter = arguments["filter"].get_document().value;
-    options::count options{};
-
-    if (arguments["collation"]) {
-        options.collation(arguments["collation"].get_document().value);
-    }
-
-    if (arguments["limit"]) {
-        options.limit(arguments["limit"].get_int32().value);
-    }
-
-    if (arguments["skip"]) {
-        options.skip(arguments["skip"].get_int32().value);
-    }
-
-    int64_t count;
-    if (client_session* session = _lookup_session(operation["arguments"].get_document().value)) {
-        // CXX-1940: use estimated count instead, once it can support filters and sessions
-        count = _coll->count_documents(*session, filter, options);
-    } else {
-        count = _coll->count_documents(filter, options);
-    }
-
-    auto result = builder::basic::document{};
-
-    // The JSON parser reads the counts as int32, so the document will not be equal to the expected
-    // result without casting.
-    result.append(builder::basic::kvp("result", static_cast<std::int32_t>(count)));
-
-    return result.extract();
-}
-
 document::value operation_runner::_run_distinct(document::view operation) {
     document::view arguments = operation["arguments"].get_document().value;
     document::view filter{};
@@ -243,7 +196,7 @@ document::value operation_runner::_run_distinct(document::view operation) {
         filter = arguments["filter"].get_document().value;
     }
 
-    bsoncxx::stdx::string_view field_name = arguments["fieldName"].get_utf8().value;
+    bsoncxx::stdx::string_view field_name = arguments["fieldName"].get_string().value;
 
     options::distinct options{};
 
@@ -301,10 +254,14 @@ document::value operation_runner::_run_find(document::view operation) {
         options.sort(arguments["sort"].get_document().value);
     }
 
+    if (arguments["allowDiskUse"]) {
+        options.allow_disk_use(arguments["allowDiskUse"].get_bool().value);
+    }
+
     if (arguments["modifiers"]) {
         document::view modifiers = arguments["modifiers"].get_document().value;
         if (modifiers["$comment"]) {
-            options.comment(modifiers["$comment"].get_utf8().value);
+            options.comment(modifiers["$comment"].get_string().value);
         }
 
         if (modifiers["$hint"]) {
@@ -360,6 +317,13 @@ document::value operation_runner::_run_delete_many(document::view operation) {
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     auto result = builder::basic::document{};
     std::int32_t deleted_count = 0;
 
@@ -390,6 +354,13 @@ document::value operation_runner::_run_delete_one(document::view operation) {
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     auto result = builder::basic::document{};
     std::int32_t deleted_count = 0;
 
@@ -420,6 +391,20 @@ document::value operation_runner::_run_find_one_and_delete(document::view operat
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     if (arguments["projection"]) {
         options.projection(arguments["projection"].get_document().value);
     }
@@ -475,13 +460,20 @@ document::value operation_runner::_run_find_one_and_replace(document::view opera
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     if (arguments["projection"]) {
         options.projection(arguments["projection"].get_document().value);
     }
 
     if (arguments["returnDocument"]) {
         std::string return_document =
-            bsoncxx::string::to_string(arguments["returnDocument"].get_utf8().value);
+            bsoncxx::string::to_string(arguments["returnDocument"].get_string().value);
 
         if (return_document == "After") {
             options.return_document(options::return_document::k_after);
@@ -528,6 +520,13 @@ document::value operation_runner::_run_find_one_and_update(document::view operat
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     if (arguments["projection"]) {
         options.projection(arguments["projection"].get_document().value);
     }
@@ -538,7 +537,7 @@ document::value operation_runner::_run_find_one_and_update(document::view operat
 
     if (arguments["returnDocument"]) {
         std::string return_document =
-            bsoncxx::string::to_string(arguments["returnDocument"].get_utf8().value);
+            bsoncxx::string::to_string(arguments["returnDocument"].get_string().value);
 
         if (return_document == "After") {
             options.return_document(options::return_document::k_after);
@@ -640,19 +639,26 @@ document::value operation_runner::_run_insert_many(document::view operation) {
 document::value operation_runner::_run_insert_one(document::view operation) {
     document::view arguments = operation["arguments"].get_document().value;
     document::view document = arguments["document"].get_document().value;
-    auto result = builder::basic::document{};
-    stdx::optional<result::insert_one> insert_one_result;
-    if (client_session* session = _lookup_session(operation["arguments"].get_document().value)) {
-        insert_one_result = _coll->insert_one(*session, document);
-    } else {
-        insert_one_result = _coll->insert_one(document);
+
+    options::insert opts{};
+    if (arguments["bypassDocumentValidation"]) {
+        opts.bypass_document_validation(true);
     }
-    types::value inserted_id{types::b_null{}};
+
+    stdx::optional<result::insert_one> insert_one_result;
+    if (client_session* session = _lookup_session(arguments)) {
+        insert_one_result = _coll->insert_one(*session, document, opts);
+    } else {
+        insert_one_result = _coll->insert_one(document, opts);
+    }
+
+    types::bson_value::view inserted_id{types::b_null{}};
 
     if (insert_one_result) {
         inserted_id = insert_one_result->inserted_id();
     }
 
+    auto result = builder::basic::document{};
     result.append(builder::basic::kvp("result", [inserted_id](builder::basic::sub_document subdoc) {
         subdoc.append(builder::basic::kvp("insertedId", inserted_id));
     }));
@@ -670,6 +676,13 @@ document::value operation_runner::_run_replace_one(document::view operation) {
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     if (arguments["upsert"]) {
         options.upsert(arguments["upsert"].get_bool().value);
     }
@@ -683,7 +696,7 @@ document::value operation_runner::_run_replace_one(document::view operation) {
     } else {
         replace_result = _coll->replace_one(filter, replacement, options);
     }
-    bsoncxx::stdx::optional<types::value> upserted_id{};
+    bsoncxx::stdx::optional<types::bson_value::view> upserted_id{};
 
     if (replace_result) {
         matched_count = replace_result->matched_count();
@@ -731,6 +744,13 @@ document::value operation_runner::_run_update_many(document::view operation) {
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
+
     if (arguments["upsert"]) {
         options.upsert(arguments["upsert"].get_bool().value);
     }
@@ -768,7 +788,7 @@ document::value operation_runner::_run_update_many(document::view operation) {
             throw std::logic_error{"update must be a document or an array"};
     }
 
-    bsoncxx::stdx::optional<types::value> upserted_id{};
+    bsoncxx::stdx::optional<types::bson_value::view> upserted_id{};
 
     if (update_result) {
         matched_count = update_result->matched_count();
@@ -816,6 +836,12 @@ document::value operation_runner::_run_update_one(document::view operation) {
         options.collation(arguments["collation"].get_document().value);
     }
 
+    if (arguments["hint"]) {
+        if (arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+            options.hint(hint{arguments["hint"].get_string().value});
+        else
+            options.hint(hint{arguments["hint"].get_document().value});
+    }
     if (arguments["upsert"]) {
         options.upsert(arguments["upsert"].get_bool().value);
     }
@@ -853,7 +879,7 @@ document::value operation_runner::_run_update_one(document::view operation) {
             throw std::logic_error{"update must be a document or an array"};
     }
 
-    bsoncxx::stdx::optional<types::value> upserted_id{};
+    bsoncxx::stdx::optional<types::bson_value::view> upserted_id{};
 
     if (update_result) {
         matched_count = update_result->matched_count();
@@ -900,17 +926,29 @@ void operation_runner::_set_collection_options(document::view operation) {
     document::view options = operation["collectionOptions"].get_document().value;
 
     if (options["writeConcern"]) {
-        document::view write_concern_options = options["writeConcern"].get_document().value;
-        int32_t w_option = write_concern_options["w"].get_int32().value;
         write_concern w;
-        w.nodes(w_option);
+        document::view write_concern_options = options["writeConcern"].get_document().value;
+
+        // Empty writeConcern document means use the default.
+        if (!write_concern_options.empty()) {
+            REQUIRE(write_concern_options["w"]);
+
+            int32_t w_option = write_concern_options["w"].get_int32().value;
+            w.nodes(w_option);
+        }
         _coll->write_concern(w);
     }
 
     if (options["readConcern"]) {
         document::view read_concern_options = options["readConcern"].get_document().value;
         read_concern rc;
-        rc.acknowledge_string(read_concern_options["level"].get_utf8().value);
+
+        // Empty readConcern document means use the default.
+        if (!read_concern_options.empty()) {
+            REQUIRE(read_concern_options["level"]);
+
+            rc.acknowledge_string(read_concern_options["level"].get_string().value);
+        }
         _coll->read_concern(rc);
     }
 }
@@ -952,11 +990,18 @@ document::value operation_runner::_run_bulk_write(document::view operation) {
     for (auto&& request_element : requests) {
         auto request = request_element.get_document().value;
         auto request_arguments = request["arguments"].get_document().value;
-        auto operation_name = request["name"].get_utf8().value;
+        auto operation_name = request["name"].get_string().value;
 
         if (operation_name.compare("updateOne") == 0) {
             auto update_one = _build_update_model<model::update_one>(request_arguments);
 
+            if (request_arguments["hint"]) {
+                if (request_arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+                    update_one.hint(hint{request_arguments["hint"].get_string().value});
+                else
+                    update_one.hint(hint{request_arguments["hint"].get_document().value});
+            }
+
             if (request_arguments["collation"]) {
                 update_one.collation(request_arguments["collation"].get_document().value);
             }
@@ -973,6 +1018,13 @@ document::value operation_runner::_run_bulk_write(document::view operation) {
         } else if (operation_name.compare("updateMany") == 0) {
             auto update_many = _build_update_model<model::update_many>(request_arguments);
 
+            if (request_arguments["hint"]) {
+                if (request_arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+                    update_many.hint(hint{request_arguments["hint"].get_string().value});
+                else
+                    update_many.hint(hint{request_arguments["hint"].get_document().value});
+            }
+
             if (request_arguments["collation"]) {
                 update_many.collation(request_arguments["collation"].get_document().value);
             }
@@ -990,6 +1042,13 @@ document::value operation_runner::_run_bulk_write(document::view operation) {
             document::view filter = request_arguments["filter"].get_document().value;
             document::view replacement = request_arguments["replacement"].get_document().value;
             model::replace_one replace_one(filter, replacement);
+            if (request_arguments["hint"]) {
+                if (request_arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+                    replace_one.hint(hint{request_arguments["hint"].get_string().value});
+                else
+                    replace_one.hint(hint{request_arguments["hint"].get_document().value});
+            }
+
             if (request_arguments["collation"]) {
                 replace_one.collation(request_arguments["collation"].get_document().value);
             }
@@ -1006,6 +1065,13 @@ document::value operation_runner::_run_bulk_write(document::view operation) {
         } else if (operation_name.compare("deleteOne") == 0) {
             document::view filter = request_arguments["filter"].get_document().value;
             model::delete_one delete_one(filter);
+            if (request_arguments["hint"]) {
+                if (request_arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+                    delete_one.hint(hint{request_arguments["hint"].get_string().value});
+                else
+                    delete_one.hint(hint{request_arguments["hint"].get_document().value});
+            }
+
             if (request_arguments["collation"]) {
                 delete_one.collation(request_arguments["collation"].get_document().value);
             }
@@ -1014,6 +1080,13 @@ document::value operation_runner::_run_bulk_write(document::view operation) {
         } else if (operation_name.compare("deleteMany") == 0) {
             document::view filter = request_arguments["filter"].get_document().value;
             model::delete_many delete_many(filter);
+            if (request_arguments["hint"]) {
+                if (request_arguments["hint"].type() == bsoncxx::v_noabi::type::k_utf8)
+                    delete_many.hint(hint{request_arguments["hint"].get_string().value});
+                else
+                    delete_many.hint(hint{request_arguments["hint"].get_document().value});
+            }
+
             if (request_arguments["collation"]) {
                 delete_many.collation(request_arguments["collation"].get_document().value);
             }
@@ -1133,7 +1206,7 @@ document::value operation_runner::_run_start_transaction(bsoncxx::document::view
         }
     }
 
-    auto session = _lookup_session(operation["object"].get_utf8().value);
+    auto session = _lookup_session(operation["object"].get_string().value);
     REQUIRE(session);
     session->start_transaction(txn_opts);
 
@@ -1141,14 +1214,14 @@ document::value operation_runner::_run_start_transaction(bsoncxx::document::view
 }
 
 document::value operation_runner::_run_commit_transaction(bsoncxx::document::view operation) {
-    auto session = _lookup_session(operation["object"].get_utf8().value);
+    auto session = _lookup_session(operation["object"].get_string().value);
     REQUIRE(session);
     session->commit_transaction();
     return bsoncxx::builder::basic::make_document();
 }
 
 document::value operation_runner::_run_abort_transaction(bsoncxx::document::view operation) {
-    auto session = _lookup_session(operation["object"].get_utf8().value);
+    auto session = _lookup_session(operation["object"].get_string().value);
     REQUIRE(session);
     session->abort_transaction();
     return bsoncxx::builder::basic::make_document();
@@ -1187,7 +1260,21 @@ document::value operation_runner::_run_configure_fail_point(bsoncxx::document::v
     return result.extract();
 }
 
-operation_runner::operation_runner(collection* coll) : _coll(coll) {}
+document::value operation_runner::_create_index(const document::view& operation) {
+    auto arguments = operation["arguments"];
+    auto session = _lookup_session(arguments.get_document().value);
+    REQUIRE(session);
+
+    auto name = arguments["name"].get_string().value;
+    auto keys = arguments["keys"].get_document().value;
+
+    bsoncxx::builder::basic::document opts;
+    opts.append(bsoncxx::builder::basic::kvp("name", name));
+
+    return _coll->create_index(*session, keys, opts.extract());
+}
+
+operation_runner::operation_runner(collection* coll) : operation_runner(nullptr, coll) {}
 operation_runner::operation_runner(database* db,
                                    collection* coll,
                                    client_session* session0,
@@ -1199,16 +1286,16 @@ document::value operation_runner::run(document::view operation) {
     using namespace bsoncxx::builder::basic;
     bsoncxx::document::value empty_document({});
 
-    stdx::string_view key = operation["name"].get_utf8().value;
+    stdx::string_view key = operation["name"].get_string().value;
     stdx::string_view object;
     if (operation["object"]) {
-        object = operation["object"].get_utf8().value;
+        object = operation["object"].get_string().value;
     }
 
     if (key.compare("aggregate") == 0) {
         return _run_aggregate(operation);
     } else if (key.compare("count") == 0) {
-        return _run_count(operation);
+        throw std::logic_error{"count command not supported"};
     } else if (key.compare("countDocuments") == 0) {
         return _run_count_documents(operation);
     } else if (key.compare("estimatedDocumentCount") == 0) {
@@ -1273,11 +1360,15 @@ document::value operation_runner::run(document::view operation) {
         }
         return empty_document;
     } else if (key.compare("rename") == 0) {
-        _coll->rename(operation["arguments"]["to"].get_utf8().value);
+        _coll->rename(operation["arguments"]["to"].get_string().value);
         return empty_document;
     } else if (key.compare("drop") == 0) {
         _coll->drop();
         return empty_document;
+    } else if (key.compare("dropCollection") == 0) {
+        auto collection_name = operation["arguments"]["collection"].get_string().value;
+        _db->collection(collection_name).drop();
+        return empty_document;
     } else if (key.compare("listCollectionNames") == 0) {
         _db->list_collection_names();
         return empty_document;
@@ -1300,6 +1391,51 @@ document::value operation_runner::run(document::view operation) {
         auto bucket = _db->gridfs_bucket();
         bucket.download_to_stream(operation["arguments"]["id"].get_value(), &null_stream);
 
+        return empty_document;
+    } else if (key.compare("createCollection") == 0) {
+        auto collection_name = operation["arguments"]["collection"].get_string().value;
+        auto session = _lookup_session(operation["arguments"].get_document().value);
+        REQUIRE(session);
+
+        _db->create_collection(*session, collection_name);
+        return empty_document;
+    } else if (key.compare("assertCollectionNotExists") == 0) {
+        auto collection_name = operation["arguments"]["collection"].get_string().value;
+        client client{uri{}};
+        REQUIRE_FALSE(client[_db->name()].has_collection(collection_name));
+        return empty_document;
+    } else if (key.compare("assertCollectionExists") == 0) {
+        auto collection_name = operation["arguments"]["collection"].get_string().value;
+        client client{uri{}};
+        REQUIRE(client[_db->name()].has_collection(collection_name));
+        return empty_document;
+    } else if (key.compare("createIndex") == 0) {
+        return _create_index(operation);
+    } else if (key.compare("assertIndexNotExists") == 0) {
+        client client{uri{}};
+        auto cursor = client[_db->name()][_coll->name()].list_indexes();
+
+        REQUIRE(cursor.end() ==
+                std::find_if(cursor.begin(),
+                             cursor.end(),
+                             [operation](bsoncxx::v_noabi::document::view doc) {
+                                 return (doc["name"].get_string() ==
+                                         operation["arguments"]["index"].get_string());
+                             }));
+
+        return empty_document;
+    } else if (key.compare("assertIndexExists") == 0) {
+        client client{uri{}};
+        auto cursor = client[_db->name()][_coll->name()].list_indexes();
+
+        REQUIRE(cursor.end() !=
+                std::find_if(cursor.begin(),
+                             cursor.end(),
+                             [operation](bsoncxx::v_noabi::document::view doc) {
+                                 return (doc["name"].get_string() ==
+                                         operation["arguments"]["index"].get_string());
+                             }));
+
         return empty_document;
     } else {
         throw std::logic_error{"unsupported operation: " + string::to_string(key)};
